#include <glib.h>

#include <vhttpd/http_message.h>

#include "private.h"

/*** helpers */

static const char _HTTP_VERSION[] = "HTTP/1.1";

static void
_print_header (gpointer key, gpointer value, gpointer data)
{
  g_string_append_printf ((GString *) data, "%s: %s%s",
			  (const char *) key, (const char *) value, _CRLF);
}

static void
_print_cookie (gpointer key, gpointer value, gpointer data)
{
  g_string_append_printf ((GString *) data, "Set-Cookie:%s=%s%s",
			  (const char *) key, (const char *) value, _CRLF);
}

/*** private API */

GString *
message_write_header (const message_t * r,const char *host)
{
  GString *data = g_string_new("");

  if (r)
  {
    g_hash_table_foreach (r->headers_, _print_header, data);
    /* "host" header is required by HTTP/1.1 */
    if ((r->type_ == message_request) && host) _print_header((gpointer)"host", (gpointer)host, data);
    g_hash_table_foreach (r->cookies_, _print_cookie, data);
  }

  g_string_append (data, _CRLF);	/* end of the headers */
  return data;
}


GString *
message_write_startline(const message_t * r)
{
  GString *data = g_string_new("");
  retcode_t code;

  if (r)
    switch (r->type_)
    {
	case message_response:
	    code = message_get_code (r);
	    g_string_printf (data, "%s %d %s%s",_HTTP_VERSION, code, http_strerror (code), _CRLF);
	    break;
	case message_request:
	    g_string_printf (data, "%s %s %s%s",message_get_method(r),message_get_uri(r),_HTTP_VERSION, _CRLF);
	    break;
    };

  return data;
}
