#include <stdlib.h>
#include <unistd.h>

#include <errno.h>

#include <glib.h>

#include <vhttpd/socket.h>
#include <vhttpd/channel.h>

#include "channel_private.h"

void
plain_channel_init (channel_t *c, int fd, char *remote_addr, int flags)
{
    c->accept = FLAG_ISSET (flags, C_SO_SERVER) ? plain_channel_accept : NULL;
    c->send = plain_channel_send;
    c->sendfile = plain_channel_sendfile;
    c->receive = plain_channel_receive;
    c->destroy = destroy_plain_channel;
    c->fd = fd;
    c->remote_addr = remote_addr;
    c->type = CHANNEL_TYPE_PLAIN;
    c->flags = flags;
}

void
destroy_plain_channel (channel_t *c)
{
  close_socket (c->fd);
  g_free (c->remote_addr);
  g_free (c);
}

int
plain_channel_send (channel_t *c, const char *buf, size_t len)
{
    return socket_send (c->fd, buf, len);
}

int
plain_channel_sendfile (channel_t *c, int fd, size_t len)
{
    return socket_sendfile (c->fd, fd, len);
}

int
plain_channel_receive (channel_t *c, buffer_t *buf)
{
    return socket_receive (c->fd, buf);
}

channel_t *
plain_channel_accept (channel_t *c)
{
  char *remote_addr = NULL;
  channel_t *ch_accepted;

  ch_accepted = g_new0 (channel_t, 1);
  int fd = socket_accept (c->fd, &remote_addr,c->flags);
  if (fd >= 0)
      plain_channel_init (ch_accepted, fd, remote_addr, 0);

  return ch_accepted;
}
