#ifndef VHTTPD_PRIVATE_H
#define VHTTPD_PRIVATE_H

#include <glib.h>

#include <vhttpd/codes.h>
#include <vhttpd/buffer.h>
#include <vhttpd/tls.h>

/*** http messages */

typedef enum
{
  parse_request_line,
  parse_headers,
  parse_body
} parse_state_t;

typedef enum
{
    message_request,
    message_response
} message_type_t;

struct _message_t
{
  /* common part */
  message_type_t type_;
  GHashTable *headers_;
  GHashTable *cookies_;
  char *body_;
  size_t length_;

  /* response part */
  retcode_t code_;
  int fd_;

  /* request part */
  int plain_;
  char *version_;
  char *method_;
  char *uri_;
  char *query_;
  char *startline_;

  /* request helpers */
  char *lastheader_;
  parse_state_t state_;
};

int message_read_startline_version (struct _message_t *r,const char *);
int message_read_startline_method (struct _message_t *r,const char *);
int message_read_startline_code (struct _message_t *r,const char *);
int message_read_startline_uri (struct _message_t *r,const char *);

char * message_read_line (buffer_t *);
retcode_t message_read_startline (struct _message_t *r, buffer_t *);
retcode_t message_read_header (struct _message_t *, buffer_t *);
retcode_t message_read_body (struct _message_t *, buffer_t *);
retcode_t message_read (struct _message_t *, buffer_t *);

GString *message_write_header(const struct _message_t *,const char *host);
GString *message_write_startline(const struct _message_t *);


extern const char _LWS[];
extern const char _CRLF[];

#endif
