#include <unistd.h>

#include <glib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <string.h>
#include <vhttpd/socket.h>
#include <vhttpd/tls.h>
#include <vhttpd/channel.h>
#include <vhttpd/tls_channel.h>
#include <vhttpd/smart_channel.h>

#include "channel_private.h"
#include "tls_channel_private.h"

/*** types and definitions */
int smart_channel_receive (channel_t *c, buffer_t *buf);
channel_t * smart_channel_accept(channel_t *c);

/*** functions */

/** First byte in SSL request reffers to SSL packet type.
    Type 0x16 means handshake request */
static int
channel_is_ssl (channel_t *c)
{
    int fd = channel_get_fd(c);
    char ch;

    return recv (fd, &ch, 1, MSG_PEEK) == 1 && ( ch == 0x16) ;
}

static void
smart_channel_switch(channel_t *c,int type)
{
    switch (type)
    {
	case CHANNEL_TYPE_PLAIN:
	    c->accept = plain_channel_accept;
	    c->send = plain_channel_send;
	    c->sendfile = plain_channel_sendfile;
	    c->receive = plain_channel_receive;
	    break;
	case CHANNEL_TYPE_TLS:
	    c->accept = tls_channel_accept;
	    c->send = tls_channel_send;
	    c->sendfile = tls_channel_sendfile;
	    c->receive = tls_channel_receive;
	    break;
	case CHANNEL_TYPE_UNKNOWN:
	    c->accept = smart_channel_accept;
	    c->send = NULL;
	    c->sendfile = NULL;
	    c->receive = smart_channel_receive;
	    break;
    }
    c->type = type;
}


channel_t *
smart_channel_accept(channel_t *c)
{
    channel_t *ch_accepted = tls_channel_accept(c);
    /* setup client's channel type to unknown */
    if (ch_accepted)
	smart_channel_switch(ch_accepted,CHANNEL_TYPE_UNKNOWN);

    return ch_accepted;
}

int
smart_channel_receive (channel_t *c, buffer_t *buf)
{
    smart_channel_switch(c, channel_is_ssl(c)?CHANNEL_TYPE_TLS:CHANNEL_TYPE_PLAIN);
    return c->receive(c, buf);
}

channel_t *
create_smart_channel_from_socket (int socket_fd, const char *addr, const char *keyfile,
                   const char *certfile, int flags)
{
    channel_t *ch = create_tls_channel_from_socket(socket_fd, addr, keyfile, certfile, flags|C_SO_SERVER);
    if (ch)
	smart_channel_switch(ch, CHANNEL_TYPE_UNKNOWN);
    return ch;
}

channel_t *
create_smart_channel (const char *addr, const char *service, const char *keyfile,
                   const char *certfile, int flags)
{
    channel_t *ch = create_tls_channel(addr,service,keyfile,certfile,flags|C_SO_SERVER);
    if (ch)
	smart_channel_switch(ch, CHANNEL_TYPE_UNKNOWN);
    return ch;
}
